Module interval

Module interval 

Useful abstractions to work with intervals

Structs

IntDegreeSet
A set of IntDegree values, efficiently implemented as a bitfield.
IntDegreeSetSetIter
Iterator returned by the iter method.
IntervalSet
A set of Interval values, efficiently implemented as a bitfield.
IntervalSetSetIter
Iterator returned by the iter method.

Enums

IntDegree
Enum representing interval degrees. It is used to calculate the correct enharmonic notes from given root.
Interval
Enum representing intervals of a chord